unit Tiles;

interface

uses
  Graphics, SysUtils, Dialogs;

const
  //Code-color Consts
  MapCodeAir = 0;       MapColorAir       = $FF00FF;//Pink-Air
  MapCodeWater = 1;     MapColorWater     = $FF0000;//Blue-Water
  MapCodeLadder= 2;     MapColorLadder    = $00FFFF;//Yellow-Ladder
  MapCodeDeath = 3;     MapColorDeath     = $0000FF;//Red-Death
  MapCodeHalfBlock = 5; MapColorHalfBlock = $808080;//Gray-HalfBlock
  MapCodeBlock = 6;     MapColorBlock     = $000000;//Black-Block

  //Other consts
  TileWidth = 64;
  TileHeight = 64;

type
  TFreemap = array[0..TileWidth - 1, 0..TileHeight - 1] of Byte; //8-bit bitmap

type
  TFileTile = record
    Bitmap: array[0..TileWidth - 1, 0..TileHeight - 1] of TColor;
    Freemap: TFreemap;
  end;

type
  TTile = record
    Bitmap: TBitmap;
    Freemap: TFreemap;
  end;

type
  TTiles = class
  private
    FTiles: array of TTile;
    FTileCount: Byte;
    function ColorToCode(AColor: TColor): Byte;
    function CodeToColor(ACode: Byte): TColor;
  public
    constructor Create;
    destructor Destroy; override;
    function LoadTiles(FileName: String): Boolean;
    function SaveTiles(FileName: String): Boolean;
    function GetBitmap(Index: Byte): TBitmap;
    function GetFreemap(Index: Byte): TFreemap;
    function AssignFreemapToBitmap(Index: Byte; DestBitmap: TBitmap): Boolean;
    procedure AddTile(ABitmap: TBitmap; AFreemap: TBitmap);
    function TileCount: Byte;
    procedure FreeBitmaps;
  end;

implementation

{ TTiles }

procedure TTiles.AddTile(ABitmap: TBitmap; AFreemap: TBitmap);
var
  I, J: Integer;
begin
  if FTileCount = High(Byte) then
    Exit;
  Inc(FTileCount);

  SetLength(FTiles, FTileCount);
  for J := 0 to TileHeight - 1 do
    for I := 0 to TileWidth - 1 do
      FTiles[FTileCount - 1].Freemap[I, J] := ColorToCode(AFreemap.Canvas.Pixels[I, J]);
  FTiles[FTileCount - 1].Bitmap := TBitmap.Create;
  FTiles[FTileCount - 1].Bitmap.Assign(ABitmap);
end;

function TTiles.CodeToColor(ACode: Byte): TColor;
begin
  case ACode of
    MapCodeAir: Result := MapColorAir;
    MapCodeWater: Result := MapColorWater;
    MapCodeLadder: Result := MapColorLadder;
    MapCodeDeath: Result := MapColorDeath;
    MapCodeHalfBlock: Result := MapColorHalfBlock;
  else
    Result := MapColorBlock;
  end;
end;

function TTiles.ColorToCode(AColor: TColor): Byte;
begin
  case AColor of
    MapColorAir: Result := MapCodeAir;
    MapColorWater: Result := MapCodeWater;
    MapColorLadder: Result := MapCodeLadder;
    MapColorDeath: Result := MapCodeDeath;
    MapColorHalfBlock: Result := MapCodeHalfBlock;
  else
    Result := MapCodeBlock;
  end;
end;

constructor TTiles.Create;
begin
  FTileCount := 0;
end;

destructor TTiles.Destroy;
var
  I: Integer;
begin
  for I := 1 to FTileCount do
  begin
    FTiles[I-1].Bitmap.Free;
    FTiles[I-1].Bitmap := nil;
  end;
  SetLength(FTiles, 0);
  FTiles := nil;
  inherited Destroy;
end;

function TTiles.GetBitmap(Index: Byte): TBitmap;
begin
  if Index < FTileCount then
    Result := FTiles[Index].Bitmap
  else
    Result := nil;
end;

function TTiles.GetFreemap(Index: Byte): TFreemap;
begin
  if Index < FTileCount then
    Result := FTiles[Index].Freemap;
end;

function TTiles.AssignFreemapToBitmap(Index: Byte; DestBitmap: TBitmap): Boolean;
var
  I, J: Integer;
  TmpBmp:  TBitmap;
begin
  TmpBmp := TBitmap.Create;
  if Index < FTileCount then
  try
    TmpBmp.PixelFormat := pf8bit;
    TmpBmp.Width := TileWidth;
    TmpBmp.Height := TileHeight;
    TmpBmp.Height := TileHeight;
    for J := 0 to TileHeight - 1do
      for I := 0 to TileWidth - 1 do
        TmpBmp.Canvas.Pixels[I, J] := CodeToColor(FTiles[Index].Freemap[I, J]);
    DestBitmap.Assign(TmpBmp);
    TmpBmp.Free;
    Result := True;
  except
    TmpBmp.Free;
    Result := False;
    Exit;
  end
  else
    Result := False;
end;

function TTiles.LoadTiles(FileName: String): Boolean;
var
  F: file of TFileTile;
  FT: TFileTile;
  I, J: Integer;
begin
  if not FileExists(FileName) then
  begin
    ShowMessage('Error during resource loading: File not found (' + FileName + ')');
    Result := False;
    Exit;
  end;

  AssignFile(F, FileName);
  Reset(F);
  while not Eof(F) do
  try
    Read(F, FT);
    Inc(FTileCount);
    SetLength(FTiles, FTileCount);
    FTiles[FTileCount - 1].Bitmap := TBitmap.Create;
    FTiles[FTileCount - 1].Bitmap.PixelFormat := pf32bit;
    FTiles[FTileCount - 1].Bitmap.Width := TileWidth;
    FTiles[FTileCount - 1].Bitmap.Height := TileHeight;
    for J := 0 to TileHeight - 1 do
      for I := 0 to TileWidth - 1 do
        FTiles[FTileCount - 1].Bitmap.Canvas.Pixels[I, J] := FT.Bitmap[I, J];
    for J := 0 to TileHeight - 1 do
      for I := 0 to TileWidth - 1 do
        FTiles[FTileCount - 1].Freemap[I, J] := FT.Freemap[I, J];
  except
    on E: Exception do
    begin
      ShowMessage('Error during resource loading: ' + E.Message);
      CloseFile(F);
      Result := False;
      Exit;
      break;
    end;
  end;
  CloseFile(F);
  Result := True;
end;

function TTiles.SaveTiles(FileName: String): Boolean;
var
  F: file of TFileTile;
  FT: TFileTile;
  I, J, K: Integer;
begin
  AssignFile(F, FileName);
  Rewrite(F);
  for K := 1 to FTileCount do
  begin
    for J := 0 to TileHeight - 1 do
      for I := 0 to TileWidth - 1 do
        Ft.Bitmap[I, J] := FTiles[K-1].Bitmap.Canvas.Pixels[I, J];
    for J := 0 to TileHeight - 1 do
      for I := 0 to TileWidth - 1 do
        Ft.Freemap[I, J] := FTiles[K-1].Freemap[I, J];
    Write(F, FT);
  end;
  CloseFile(F);
  Result := True;
end;

function TTiles.TileCount: Byte;
begin
  Result := FTileCount;
end;

procedure TTiles.FreeBitmaps;
var
  I: Integer;
begin
  for I := 0 to FTileCount - 1 do
  begin
    FTiles[I].Bitmap.Free;
    FTiles[I].Bitmap := nil;
  end;
end;

end.
